/**
 * @file util.cpp
 *
 * @author Jan Hadrava
 * @author Petr Cermak
 *
 * @ingroup iopccommon
 */

#include "util.h"
#include <iostream>
#include <ostream>
#include <istream>
#include <sstream>
#include <vector>
#include "exceptionBase.h"
#include "trace.h"
#include <ctime>
#include <iomanip>

using namespace std;

namespace iopc {

//-----------------------------------------------------------------------
//
//  File:    StringUtils.cpp
//
//  Purpose:  STL split string utility
//  Author:    Paul J. Weiss
//
//------------------------------------------------------------------------
int StringUtil::splitString(const string& input,
  const string& delimiter, vector<string>& results, bool skipEmpty)
{
	int iPos = -1;
	int newPos = -1;
	int sizeS2 = delimiter.size();
	int isize = input.size();

	vector<int> positions;

	newPos = input.find (delimiter, 0);

	if( newPos < 0 ) {
		results.push_back(input);
		return 1;
	}

	int numFound = 0;

	while( newPos > iPos )
	{
		numFound++;
		positions.push_back(newPos);
		iPos = newPos;
		newPos = input.find (delimiter, iPos+sizeS2/*+1*/);
	}

	for( int i=0; i <= (int)positions.size(); i++ )
	{
		string s;
		if( i == 0 ) {
			s = input.substr( 0, positions[i] );
		} else {
			int offset = positions[i-1] + sizeS2;
			if( offset < isize )
			{
				if( i == (int)positions.size() )
				{
					s = input.substr(offset);
				}
				else if( i > 0 )
				{
					s = input.substr( positions[i-1] + sizeS2,
						positions[i] - positions[i-1] - sizeS2 );
				}
			}
		}
		if (!skipEmpty || s.size() > 0)
		{
			results.push_back(s);
		}
	}
	return results.size();
}

string StringUtil::joinStrings(vector<string> strings, const string& delimiter)
{
	ostringstream os;
	unsigned int pos = 1;
	for (vector<string>::iterator i = strings.begin(); i != strings.end(); i++, pos++) {
		os << (*i);
		if (pos != strings.size()) {
			os << delimiter;
		}
	}
	return os.str();
}

map<string, string> StringUtil::createParamMap(const string& paramStr)
{
	map<string, string> paramMap;
	vector<string> paramList;

	StringUtil::splitString(paramStr, IOPC_PARAM_DELIMITER, paramList);

	for (vector<string>::iterator i = paramList.begin(); i != paramList.end(); i++) {
		string str = (*i);
		unsigned int pos = str.find_first_of('=');
		if (pos == string::npos || pos == 0 || pos == str.size() - 1) {
			IOPC_LOG_THROW(IopcIncorrectUsageException("Wrong format of parameter. Name-value pair must be of name=value format."));
		}
		paramMap[str.substr(0, pos)] = str.substr(pos + 1, str.size() - pos - 1);
	}

	return paramMap;
}

string StringUtil::join(const char* string1, const char* string2)
{
	ostringstream os;

	os << string1 << string2;
	return os.str();
}

int StringUtil::parseInt(const std::string& input)
{
	istringstream is(input);
	int retval = 0;
	is >> retval;
	return retval;
}

string StringUtil::toString(int i)
{
	ostringstream os;
	os << i;
	return os.str();
}

bool StringUtil::parseBool(const std::string& input)
{
	return input == "true";
}


string Util::currentTime()
{
	time_t curr;
	tm local;
	time(&curr);
	localtime_r(&curr, &local);

	ostringstream os;
	os.width(2);
	os.fill('0');
	os << (local.tm_year % 100);
	os.width(2);
	os.fill('0');
	os << (local.tm_mon + 1);
	os.width(2);
	os.fill('0');
	os << local.tm_mday << " " ;
	os.width(2);
	os.fill('0');
	os << local.tm_hour << ":";
	os.width(2);
	os.fill('0');
	os << local.tm_min << ":";
	os.width(2);
	os.fill('0');
	os << local.tm_sec;
	return os.str();
}

}
