/**
 * @file oracle10gTypeMappingFeature.cpp
 *
 * @author Petr Cermak
 */

#include "oracle10gTypeMappingFeature.h"

#include <string>
#include <sstream>
#include "iopcmeta/type.h"
#include "iopcmeta/typeDesc.h"
#include "oci.h"
#include "ociap.h"
#include "iopcmeta/enhancedNumber.h"
#include "iopcmeta/enhancedString.h"

using namespace iopc;
using namespace std;


void Oracle10gTypeMappingFeature::initTypes()
{
	if (!TypeDesc<bool>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<bool>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(1)");
		TypeDesc<bool>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<unsigned char>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<unsigned char>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(3)");
		TypeDesc<unsigned char>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<signed char>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<signed char>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(3)");
		TypeDesc<signed char>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<short>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<short>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(5)");
		TypeDesc<short>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<unsigned short>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<unsigned short>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(5)");
		TypeDesc<unsigned short>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<int>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<int>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(10)");
		TypeDesc<int>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<unsigned int>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<unsigned int>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(10)");
		TypeDesc<unsigned int>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<long>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<long>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(10)");
		TypeDesc<long>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<unsigned long>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<unsigned long>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(10)");
		TypeDesc<unsigned long>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<float>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<float>::getType()["db.oracle.type.sql"].setStringValue("NUMBER");
		TypeDesc<float>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<double>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<double>::getType()["db.oracle.type.sql"].setStringValue("NUMBER");
		TypeDesc<double>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<long double>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<long double>::getType()["db.oracle.type.sql"].setStringValue("NUMBER");
		TypeDesc<long double>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<char>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<char>::getType()["db.oracle.type.sql"].setStringValue("CHAR(1)");
		TypeDesc<char>::getType()["db.oracle.type.sql.param"].setStringValue("CHAR");
	}
	if (!TypeDesc<wchar_t>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<wchar_t>::getType()["db.oracle.type.sql"].setStringValue("CHAR(1)");
		TypeDesc<wchar_t>::getType()["db.oracle.type.sql.param"].setStringValue("CHAR");
	}
	// Enhanced types
	if (!TypeDesc<EInt>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EInt>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(10)");
		TypeDesc<EInt>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<EUInt>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EUInt>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(10))");
		TypeDesc<EUInt>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<EChar>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EChar>::getType()["db.oracle.type.sql"].setStringValue("CHAR(1)");
		TypeDesc<EChar>::getType()["db.oracle.type.sql.param"].setStringValue("CHAR");
	}
	if (!TypeDesc<EUChar>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EUChar>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(3)");
		TypeDesc<EUChar>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<ESChar>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<ESChar>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(3)");
		TypeDesc<ESChar>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<EShort>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EShort>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(5)");
		TypeDesc<EShort>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<EUShort>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EUShort>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(5)");
		TypeDesc<EUShort>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<ELong>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<ELong>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(10)");
		TypeDesc<ELong>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<EULong>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EULong>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(10)");
		TypeDesc<EULong>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<EFloat>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EFloat>::getType()["db.oracle.type.sql"].setStringValue("NUMBER");
		TypeDesc<EFloat>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<EDouble>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EDouble>::getType()["db.oracle.type.sql"].setStringValue("NUMBER");
		TypeDesc<EDouble>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<ELDouble>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<ELDouble>::getType()["db.oracle.type.sql"].setStringValue("NUMBER");
		TypeDesc<ELDouble>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}
	if (!TypeDesc<EWCharT>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EWCharT>::getType()["db.oracle.type.sql"].setStringValue("CHAR(1)");
		TypeDesc<EWCharT>::getType()["db.oracle.type.sql.param"].setStringValue("CHAR");
	}
	if (!TypeDesc<EBool>::getType().hasMeta("db.oracle.type.sql")) {
		TypeDesc<EBool>::getType()["db.oracle.type.sql"].setStringValue("NUMBER(1)");
		TypeDesc<EBool>::getType()["db.oracle.type.sql.param"].setStringValue("NUMBER");
	}

	if (!TypeDesc<string>::getType().hasMeta("db.oracle.type.sql.param")) {
		TypeDesc<string>::getType()["db.oracle.type.sql.param"].setStringValue("VARCHAR2");
	}
	if (!TypeDesc<wstring>::getType().hasMeta("db.oracle.type.sql.param")) {
		TypeDesc<wstring>::getType()["db.oracle.type.sql.param"].setStringValue("VARCHAR2");
	}
	if (!TypeDesc<EString>::getType().hasMeta("db.oracle.type.sql.param")) {
		TypeDesc<EString>::getType()["db.oracle.type.sql.param"].setStringValue("VARCHAR2");
	}
	if (!TypeDesc<EWString>::getType().hasMeta("db.oracle.type.sql.param")) {
		TypeDesc<EWString>::getType()["db.oracle.type.sql.param"].setStringValue("VARCHAR2");
	}


	TypeDesc<bool>::getType()["db.oracle.type.otype"].setIntValue(SQLT_INT);
	TypeDesc<unsigned char>::getType()["db.oracle.type.otype"].setIntValue(SQLT_UIN);
	TypeDesc<signed char>::getType()["db.oracle.type.otype"].setIntValue(SQLT_INT);
	TypeDesc<short>::getType()["db.oracle.type.otype"].setIntValue(SQLT_INT);
	TypeDesc<unsigned short>::getType()["db.oracle.type.otype"].setIntValue(SQLT_UIN);
	TypeDesc<int>::getType()["db.oracle.type.otype"].setIntValue(SQLT_INT);
	TypeDesc<unsigned int>::getType()["db.oracle.type.otype"].setIntValue(SQLT_UIN);
	TypeDesc<long>::getType()["db.oracle.type.otype"].setIntValue(SQLT_INT);
	TypeDesc<unsigned long>::getType()["db.oracle.type.otype"].setIntValue(SQLT_UIN);
	TypeDesc<float>::getType()["db.oracle.type.otype"].setIntValue(SQLT_FLT);
	TypeDesc<double>::getType()["db.oracle.type.otype"].setIntValue(SQLT_FLT);
	TypeDesc<long double>::getType()["db.oracle.type.otype"].setIntValue(SQLT_FLT);
	TypeDesc<char>::getType()["db.oracle.type.otype"].setIntValue(SQLT_INT);
	TypeDesc<wchar_t>::getType()["db.oracle.type.otype"].setIntValue(SQLT_INT);
	TypeDesc<string>::getType()["db.oracle.type.otype"].setIntValue(SQLT_STR);
	TypeDesc<wstring>::getType()["db.oracle.type.otype"].setIntValue(SQLT_STR);
	// No need to add enhanced types as they are externalised as one of the basic types.
}

string Oracle10gTypeMappingFeature::getTypeSql(const Type& type, const MetadataHolder& attrMeta)
{
	IOPC_FUNC_INVOKED;
	if (type.getDataTypeClass() == Type::IOPC_TYPECLASS_COMPLEX) {
		IOPC_LOG_THROW(IopcIncorrectUsageException("The type parameter can't be a complex type"));
	}

	if ((type.getDataTypeClass() == Type::IOPC_TYPECLASS_STRING ||
		type.getDataType() == Type::IOPC_TYPE_EWSTRING ||
		type.getDataType() == Type::IOPC_TYPE_ESTRING) && !type.hasMeta("db.oracle.type.sql")) {

		ostringstream str;
		if (attrMeta.hasMeta("db.type.length")) {
			str << "VARCHAR2(" << attrMeta.getMeta("db.type.length").getIntValue() << ")";
		} else {
			str << "VARCHAR2(" << type.getMeta("db.type.length").getIntValue() << ")";
		}
		return str.str();
	} else {
		//IOPC_DEBUGLOG(type.getQualifiedName() + " XXXXXX");
		return type.getMeta("db.oracle.type.sql");
	}

	IOPC_LOG_THROW(IopcIncorrectUsageException("Unknown datatype"));
}
