/**
 * @file statementCache.h
 *
 * @ingroup iopclib
 *
 * @author Petr Cermak
 */

#ifndef STATEMENTCACHE_H_
#define STATEMENTCACHE_H_


#include "iopcdb/dbFeature.h"
#include "iopccommon/metadataHolder.h"
#include "iopccommon/rwLock.h"
#include <vector>

namespace iopc {

/**
 * @brief Once an SQL statement is generated, this facility allows to store it under a name and recover it later
 *
 * This class is thread safe
 */
class StatementCache
{
private:
	std::map<std::string, std::string> statementCache;
	RWLock statementCacheLock;
public:
	bool hasCachedStatement(const std::string& statementType);
	std::string getCachedStatement(const std::string& statementType);
	void setCachedStatement(const std::string& statementType, const std::string& statement);

	// Statement type constants for convenience.
	// These string constants should be appended after the qualifiedTypeName as the statement type.

	static std::string insertStatementType;
	static std::string updateStatementType;
	static std::string deleteStatementType;
	static std::string selectTableStatementType;
	static std::string selectViewStatementType;
	static std::string selectSerialIdStatementType;
	static std::string selectNextSerialIdStatementType;
	static std::string selectTypeStatementType;
	static std::string selectNextOidStatementType;
};

}

#endif /*STATEMENTCACHE_H_*/
