#ifndef ORMAPPINGSTRATEGY_H_
#define ORMAPPINGSTRATEGY_H_

#include "mappingStrategy.h"
#include "iopcmeta/type.h"
#include "dbType.h"

namespace iopc {

class Attribute;

class ORMappingStrategy : public MappingStrategy
{
private:
	DbType* dbType;
public:
	ORMappingStrategy(DbType* dbType) : dbType(dbType) {};
	virtual void init();
	virtual void postInit();
	virtual void loadOne(Connection& conn, DatabaseObject* obj, const KeyValues& keyVals, bool wait, bool lock);
	virtual OidObject* viewLoadAll(Connection& conn, const OidType& oid, bool wait, bool lock);
	virtual void updateOne(Connection& conn, DatabaseObject& obj);
	virtual void deleteOne(Connection& conn, const OidType& oid);
	virtual void deleteOne(Connection& conn, const KeyValues& keyVals);
	virtual void insertOne(Connection& conn, DatabaseObject& obj);
	virtual inline DbType& getDbType() const {return *dbType; }
	virtual inline const Type& getType() const { return dbType->getType(); }
public:


	enum ColSource {
		COLSOURCE_ATTR =0,
		COLSOURCE_INHERITEDPK,
		COLSOURCE_INHERITED,
		COLSOURCE_FILTERED
	};

	typedef std::vector<std::pair<const Attribute*, ColSource> > MappedAttributesList;
	typedef MappedAttributesList::const_iterator MappedAttributesIterator;

	typedef std::set<const DbType*> FilteredTypesList;
	typedef FilteredTypesList::const_iterator FilteredTypesIterator;
public:
	inline const MappedAttributesList& getMappedAttributes() const { return mappedAttributes; }
	inline bool hasDbTable() const { return hasTable; }
	inline DbType& getInsertToType() const { return *insertToType; }

private:


	/** List of all attributes that relate to this type. This covers:
	 * <ul>
	 * <li>own columns</li>
	 * <li>inherited columns if mapping is horizontal</li>
	 * <li>filtered columns from types with filtered mapping</li>
	 * <li>a virtual oid column</li>
	 * </ul>
	 */
	MappedAttributesList mappedAttributes;

	FilteredTypesList filteredTypes;

	void addFilteredType(DbType& type);

	bool hasFilteredColumns;
	bool hasTable;
	DbType* insertToType;
};

}

#endif /*ORMAPPINGSTRATEGY_H_*/
