/**
 * @file classGenerator.cpp
 *
 * @author Petr Cermak
 */

#include "classGenerator.h"
#include "xmlSupport.h"
#include "iopccommon/trace.h"
#include "iopccommon/util.h"
#include <cctype>
#include <algorithm>
#include <list>
#include <set>

using namespace std;

namespace iopc {

void ClassGenerator::init()
{
	IOPC_DEBUGLOG("Creating hashtable of id to element mappings");

	list<DOMElement*> objectClasses;

	for (DOMNode* child = root->getFirstChild(); child != NULL; child=child->getNextSibling()) {
		if (child->getNodeType() != DOMNode::ELEMENT_NODE) {
			continue;
		}

		DOMElement* element = (DOMElement*) child;
		idToElementMap[TOSTRING((element)->getAttribute(TOXML("id")))] = (DOMElement*) child;

		if (string(TOSTRING(element->getTagName())) == "Namespace" && string(TOSTRING(element->getAttribute(TOXML("name")))) == "iopc") {
			// locate the IOPC namespace
			nsIopcID = TOSTRING(element->getAttribute(TOXML("id")));
		} else if (string(TOSTRING(element->getTagName())) == "Class") {
			// locate all classes
			allClasses.push_back(element);
			// locate all classes of the name "Object"
			if (string(TOSTRING(element->getAttribute(TOXML("name")))) == "Object") {
				objectClasses.push_back(element);
			}
		}
	}

	/*IOPC_DEBUGLOG("Searching for id of the iopc napespace...");
	DOMNodeList* elements = root->getElementsByTagName(TOXML("Namespace"));
	for (XMLSize_t i = 0; i < elements->getLength(); i++) {
		DOMElement* element = (DOMElement*) elements->item(i);
		if (string(TOSTRING(element->getAttribute(TOXML("name")))) == "iopc") {
			nsIopcID = TOSTRING(element->getAttribute(TOXML("id")));
			break;
		}
	}*/

	IOPC_DEBUGLOG("Searching for id of the Object class...");

	// locate the only "Object" class in the namespace iopc
	for (list<DOMElement*>::iterator i = objectClasses.begin(); i != objectClasses.end(); i++) {
		if (string(TOSTRING((*i)->getAttribute(TOXML("context")))) == nsIopcID) {
			classObjectID = TOSTRING((*i)->getAttribute(TOXML("id")));
			break;
		}
	}

	/*elements = root->getElementsByTagName(TOXML("Class"));
	for (XMLSize_t i = 0; i < elements->getLength(); i++) {
		DOMElement* element = (DOMElement*) elements->item(i);
		if (string(TOSTRING(element->getAttribute(TOXML("context")))) == nsIopcID &&
			string(TOSTRING(element->getAttribute(TOXML("name")))) == "Object") {
			classObjectID = TOSTRING(element->getAttribute(TOXML("id")));
			break;
		}
	}*/

	/*IOPC_DEBUGLOG("Searching for id of the file being processed...");
	elements = root->getElementsByTagName(TOXML("File"));
	for (XMLSize_t i = 0; i < elements->getLength(); i++) {
		DOMElement* element = (DOMElement*) elements->item(i);
		if (string(TOSTRING(element->getAttribute(TOXML("name")))) == fileName) {
			fileID = TOSTRING(element->getAttribute(TOXML("id")));
			break;
		}
	}*/

}

void ClassGenerator::getClasses()
{
	IOPC_DEBUGLOG("Searching classes in the file being processed... ");
	/*for (std::map<std::string, DOMElement*>::iterator it = idToElementMap.begin(); it !=  idToElementMap.end(); it++) {

	}*/

	// locate all descendants of the Object class
	for (vector<DOMElement*>::iterator i = allClasses.begin(); i != allClasses.end(); i++) {
		if (TOSTRING((*i)->getAttribute(TOXML("context"))) != nsIopcID && isObject(*i)) {
			attributeTypes.insert(getTypeName(*i));
			IOPC_DEBUGLOG(getTypeName(*i));
			iopcClasses.push_back(*i);
		}
	}
	attributeTypes.insert("iopc::DatabaseObject");
	attributeTypes.insert("iopc::OidObject");
	attributeTypes.insert("int");
	attributeTypes.insert("unsigned int");
	attributeTypes.insert("bool");
	attributeTypes.insert("unsigned char");
	attributeTypes.insert("signed char");
	attributeTypes.insert("short int");
	attributeTypes.insert("unsigned short int");
	attributeTypes.insert("long");
	attributeTypes.insert("unsigned long");
	attributeTypes.insert("float");
	attributeTypes.insert("double");
	attributeTypes.insert("long double");
	attributeTypes.insert("char");
	attributeTypes.insert("wchar_t");
	attributeTypes.insert("std::string");
	attributeTypes.insert("std::wstring");
	attributeTypes.insert("iopc::EInt");
	attributeTypes.insert("iopc::EUInt");
	attributeTypes.insert("iopc::EChar");
	attributeTypes.insert("iopc::EUChar");
	attributeTypes.insert("iopc::ESChar");
	attributeTypes.insert("iopc::EShort");
	attributeTypes.insert("iopc::EUShort");
	attributeTypes.insert("iopc::ELong");
	attributeTypes.insert("iopc::EULong");
	attributeTypes.insert("iopc::EFloat");
	attributeTypes.insert("iopc::EDouble");
	attributeTypes.insert("iopc::ELDouble");
	attributeTypes.insert("iopc::EWCharT");
	attributeTypes.insert("iopc::EBool");
	attributeTypes.insert("iopc::EString");
	attributeTypes.insert("iopc::EWString");
	attributeTypes.insert("iopc::DbPtrBase");


	/*DOMNodeList* elements = root->getElementsByTagName(TOXML("Class"));
	for (XMLSize_t i = 0; i < elements->getLength(); i++) {
		DOMElement* element = (DOMElement*) elements->item(i);
		if (TOSTRING(element->getAttribute(TOXML("context"))) != nsIopcID &&
			isObject(element)) {
			iopcClasses.push_back(element);
			IOPC_DEBUGLOG(TOSTRING(element->getAttribute(TOXML("name"))));
		}
	}*/

}

bool ClassGenerator::isObject(DOMElement* classElement)
{
	static int yes = 1;
	static int no = 2;

	// already visited class
	if (classElement->getUserData(TOXML("x")) == &yes) {
		return true;
	}
	if (classElement->getUserData(TOXML("x")) == &no) {
		return false;
	}

	string basesStr = TOSTRING(classElement->getAttribute(TOXML("bases")));
	vector<string> bases;
	StringUtil::splitString(basesStr, " ", bases, true);

	for(vector<string>::iterator i = bases.begin(); i != bases.end(); i++) {
		// One of ancestors is of type Object
		if ((*i) == classObjectID) {
			return true;
		}

		// ignore all private and protected ancestors (they begin with protected: or private:)
		if ((*i)[0] != '_') {
			continue;
		}

		DOMElement* el = idToElementMap[*i];
		if  (isObject(el)) {
			classElement->setUserData(TOXML("x"), &yes, NULL);
			//isObjectSet.insert(classElement);
			return true;
		}
	}
	classElement->setUserData(TOXML("x"), &no, NULL);
//	isNotObjectSet.insert(classElement);
	return false;
}

// TODO: otestovat - vic ns, zadny ns.
std::string ClassGenerator::getNamespaces(DOMElement* e)
{
	string strNs;
	DOMElement* ns = idToElementMap[TOSTRING(e->getAttribute(TOXML("context")))];

//	IOPC_DEBUGLOG(TOSTRING(ns->getTagName()));
	while (ns && (string(TOSTRING(ns->getTagName())) == "Namespace")) {
		string tmpStrNs = TOSTRING(ns->getAttribute(TOXML("name")));
		if (strNs == "") {
			strNs = tmpStrNs;
		} else {
			if (tmpStrNs != "::") {
				strNs = tmpStrNs + "::" + strNs;
			}
		}
		ns = idToElementMap[TOSTRING(ns->getAttribute(TOXML("context")))];
	}
	if (strNs == "::") {
		strNs = "";
	}
	return strNs;
}


void ClassGenerator::processMembers(DOMElement* classNode, std::vector<DOMElement*>& attributes, bool& hasIopcInit)
{
	vector<string> memberIDs;
	StringUtil::splitString(TOSTRING(classNode->getAttribute(TOXML("members"))), " ", memberIDs, true);

	hasIopcInit = false;
	for (vector<string>::iterator i= memberIDs.begin(); i != memberIDs.end(); i++) {
		DOMElement* element = idToElementMap[*i];
		string tagName = TOSTRING(element->getTagName());
		if (tagName == "Field") {
			attributes.push_back(element);
		} else if (tagName == "Method") {
			// ...
			if (string(TOSTRING(element->getAttribute(TOXML("name")))) == "iopcInit" &&
				string(TOSTRING(element->getAttribute(TOXML("static")))) == "1") {
				hasIopcInit = true;
			}
		}
	}
}

string ClassGenerator::createQualifiedName(string typeName, string ns)
{
	return ns + "::" + typeName;
}

string ClassGenerator::getTypeName(DOMElement* element) {
	if (string(TOSTRING(element->getTagName())) == "FundamentalType") {
		return TOSTRING(element->getAttribute(TOXML("name")));
	}
	string ns = getNamespaces(element);
	return createQualifiedName(TOSTRING(element->getAttribute(TOXML("name"))), getNamespaces(element));
}

string ClassGenerator::getAttributeTypeName(DOMElement* element)
{
	return getTypeName(idToElementMap[TOSTRING(element->getAttribute(TOXML("type")))]);
}

string ClassGenerator::processTypeExceptions(string typeName)
{
	// DbPtr<...>
	if (typeName.find("iopc::DbPtr") == 0) {
		return string("iopc::DbPtrBase");
	}
	return typeName;
}

bool ClassGenerator::isAllowedAttributeType(string typeName)
{
	/*IOPC_DEBUGLOG(string("Testing:  ") + typeName);
	if (attributeTypes.find(typeName) == attributeTypes.end()) {
		IOPC_DEBUGLOG("false;");
	}*/
	return attributeTypes.find(typeName) != attributeTypes.end();
}

bool ClassGenerator::writeHeader(ostream& o)
{
	IOPC_DEBUGLOG("Writing headers... ");
	o << "#include \"iopcmeta/metaSupport.h\"" << endl;
	set<string> includes;

	// generate includes
	for(vector<DOMElement*>::iterator i = iopcClasses.begin(); i != iopcClasses.end(); i++) {
		string include = TOSTRING(idToElementMap[TOSTRING((*i)->getAttribute(TOXML("file")))]->getAttribute(TOXML("name")));
		if (includes.find(include) == includes.end()) {
			o << "#include \"" << include << "\"" << endl;
			includes.insert(include);
		}
	}

	bool privWarning = false;
	for(vector<DOMElement*>::iterator i = iopcClasses.begin(); i != iopcClasses.end(); i++) {
		string className = TOSTRING((*i)->getAttribute(TOXML("name")));
		string nameSpace = getNamespaces(*i);
		string qualifiedClassName = createQualifiedName(className, nameSpace);

		o << "META_BLOCK1(" << qualifiedClassName + ")" << endl;

		// Bases
		vector<string> bases;
		StringUtil::splitString(TOSTRING((*i)->getAttribute(TOXML("bases"))), " ", bases, true);
		bool moreBases = false;
		for(vector<string>::iterator j = bases.begin(); j != bases.end(); j++) {
			// ignore all private and protected ancestors (they begin with protected: or private:)
			if ((*j)[0] != '_') {
				if (!privWarning) {
					privWarning = true;
					LogWriter::logWarning("Private, protected bases are ignored.");
				}
				continue;
			}
			DOMElement* parentClass = idToElementMap[*j];
			if  (isObject(parentClass) || *j == classObjectID) {
				/*if (moreBases) {
					LogWriter::logError("Class " + qualifiedClassName + " inherits from more bases that are descendants of iopc::Object. A descendant of iopc::Object can have only one base that is descendant of the iopc::Object class.");
					return false;
				}*/
				string qcn = createQualifiedName(TOSTRING(parentClass->getAttribute(TOXML("name"))), getNamespaces(parentClass));
				o << "META_PARENT1(" << qcn << ")" << endl;
				moreBases = true;
			}
		}

		vector<DOMElement*> attributes;
		bool isIopcInit = false;
		processMembers(*i, attributes, isIopcInit);

		// attributes 1
		for(vector<DOMElement*>::iterator j = attributes.begin(); j != attributes.end(); j++) {
			string access = TOSTRING((*j)->getAttribute(TOXML("access")));
			transform(access.begin(), access.end(), access.begin(), (int(*)(int)) toupper);
			string attrType = processTypeExceptions(getAttributeTypeName(*j));
			if (!isAllowedAttributeType(attrType)) continue;
			o << "META_ATTR1(" << TOSTRING((*j)->getAttribute(TOXML("name"))) << ", " << attrType << ", Attribute::VISIBILITY_" << access << ")" << endl;
		}

		o << "META_BLOCK2(" << qualifiedClassName + ")" << endl;

		// attributes 2
		for(vector<DOMElement*>::iterator j = attributes.begin(); j != attributes.end(); j++) {
			string attrType = processTypeExceptions(getAttributeTypeName(*j));
			if (!isAllowedAttributeType(attrType)) continue;
			o << "META_ATTR2(" << qualifiedClassName << ", " << TOSTRING((*j)->getAttribute(TOXML("name"))) << ")" << endl;
		}

		o << "META_BLOCK3(" << qualifiedClassName << ", " << className << ", " << nameSpace << ")" << endl;
		if (isIopcInit) {
			o << "META_INIT(" << qualifiedClassName << ")" << endl;
		}
		o << "META_BLOCK4" << endl;
		o << endl;

	}
	return true;
}

void ClassGenerator::writeCpp(ostream& o, const string& includeFile)
{
	IOPC_DEBUGLOG("Writing cpp... ");
	o << "#include \"" << includeFile << "\"" << endl;
	for(vector<DOMElement*>::iterator i = iopcClasses.begin(); i != iopcClasses.end(); i++) {
		string className = TOSTRING((*i)->getAttribute(TOXML("name")));
		string nameSpace = getNamespaces(*i);
		string qualifiedClassName = createQualifiedName(className, nameSpace);
		o << "META_REGISTER_TYPE(" << qualifiedClassName << ")" << endl;
	}
}


}
