/**
 * @file classGenerator.h
 *
 * @author Petr Cermak
 */

#ifndef GENERATOR_H_
#define GENERATOR_H_

#include "iopccommon/defs.h"
#include "xmlSupport.h"
//#include <xercesc/dom/DOM.hpp>
#include <xercesc/dom/DOM.hpp>
#include <map>
#include <vector>
#include <ostream>
#include <set>

XERCES_CPP_NAMESPACE_USE;

namespace iopc {

class ClassGenerator {

public:
	ClassGenerator(DOMDocument* document, std::string fileName) : document(document), root(document->getDocumentElement()), fileName(fileName) {};

	void init();

	void getClasses();

	bool writeHeader(std::ostream& o);
	void writeCpp(std::ostream& o, const std::string& includeFile);

private:
	bool isObject(DOMElement* classElement);
	std::string getNamespaces(DOMElement* e);
	void processMembers(DOMElement* classNode, std::vector<DOMElement*>& attributes, bool& hasIopcInit);
	std::string createQualifiedName(std::string typeName, std::string ns);
	std::string getTypeName(DOMElement* element);
	std::string getAttributeTypeName(DOMElement* element);
	std::string processTypeExceptions(std::string typeName);
	bool isAllowedAttributeType(std::string typeName);
private:
	DOMDocument* document;
	DOMElement* root;
	std::map<std::string, DOMElement*> idToElementMap;
	std::vector<DOMElement*> allClasses;
	std::vector<DOMElement*> iopcClasses;
	std::set<std::string> attributeTypes;
	std::string fileName;

	std::string nsIopcID;
	std::string classObjectID;
	//std::string fileID;
};

}

#endif /*GENERATOR_H_*/
