#include <string>
#include <iostream>
#include <vector>

#include "iopclib/iopcHeaders.h"
#include "school.h"
#include "iopcdb/sqlStatementsFeature.h"
#include "iopclib/cache/voidCache.h"
#include "iopclib/cache/arcCacheST.h"
#include "iopclib/cache/lruCacheST.h"
#include "iopccommon/util.h"

//using namespace iopc;
using namespace std;
using namespace iopc;


void setupDB(Connection* conn)
{
	vector<string> script;

	// drop script
	script = ScriptsGenerator::getDbDropScript(conn->getDriver());

	for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
		//LogWriter::logInfo(*it);
		try {
			cout << *it << endl;
			conn->sqlNonQuery(*it);
		} catch (...) {}
	}

	// create script
	script = ScriptsGenerator::getDbCreateScript(conn->getDriver());

	for(vector<string>::const_iterator it = script.begin(); it != script.end(); it++) {
		cout << *it << endl;
		conn->sqlNonQuery(*it);
		//LogWriter::logInfo(*it);
	}
}

int main(int argc, char *argv[])
{

	try {
		Iopc::init(new TextFileMetadataLoader("metadata.config"), LogWriter::WARNING | LogWriter::ERROR, LogWriter::NONE /* | LogWriter::INFO_SQL | LogWriter::DEBUG | LogWriter::TRACE | LogWriter::INFO*/);
		cout << "Start" << endl;


		Database* db = NULL;
		CachedConnection* conn = NULL;

		try {
			//db = new CachedDatabase(DriverManager::getDriver("IopcOracle10g").getDatabase("XE"), new ComposedCache());
			//VoidCache* v = new VoidCache();
			db = new CachedDatabase(DriverManager::getDriver("IopcOracle10g").getDatabase("XE"), new ComposedCache());
			conn = (CachedConnection*)db->getConnection("username=iopc;password=iopc;autocommit=false");
			CacheKeeper cacheKeeper(MINLONG, 500000, 900000);
			cacheKeeper.setCache(conn, new HashLruCache());
			cacheKeeper.setStrategy(conn, Strategy::lazy);
			cacheKeeper.runMaintainer();
			conn->open();

			setupDB(conn);

			cout << Util::currentTime() << endl;
			for (int i = 1; i <= 10000; i++) {
				DbPtr<Person> person;
				person->name = "Mary Major";
				person->age = 60;
				person.bePersistent(conn);

				DbPtr<Employee> employee;
				employee->name = "Ola Nordmann";
				employee->age = 45;
				employee->salary = i;
				employee.bePersistent(conn);

				DbPtr<Student> student;
				student->name = "Richard Doe";
				student->age = 22;
				student->studcardid = "WCD-3223";
				//student->supervisor = employee;
				student.bePersistent(conn);;

				DbPtr<PhdStudent> phd;
				phd->name = "Joe Bloggs";
				phd->age = 27;
				phd->studcardid = "PHD-1234";
				phd->scholarship = i;
				phd.bePersistent(conn);
				if (i % 1000 == 0) cout << i << endl;
			}
			cout << Util::currentTime() << endl;
			conn->commit();
			cout << Util::currentTime() << endl;

			cout << "Persons age >= 10" << endl;
			FreeQuery q2("$::Person$ WHERE $::Person::age$ >= 30");
			Result<Person> r(conn, q2);
			cout << Util::currentTime() << endl;
			r.open();
			cout << Util::currentTime() << endl;
			int j = 0;
			int q = 0;
			LogWriter::getInstance().traceMaskTTY = LogWriter::INFO_SQL;
			for (Result<Person>::iterator it = r.begin(); it != r.end(); it++) {
				j++;
				q += it->age;
			}
			cout << Util::currentTime() << endl;
			cout << j << " " << q << endl;
			r.close();
			//cacheKeeper.stopMaintainer();
			conn->close();
		} catch (IopcException& e) {
			cout << e.what() << endl;
		}

		if (db && conn) db->returnConnection(conn);
		if (db) DriverManager::returnDatabase(db);


		cout << "end" << endl;
		Iopc::shutdown();
	} catch (IopcException& e) {
		cout << e.what();
	}
	cin.getline(NULL, 0);
	return 0;
}


